/*-
 * Copyright (c) 2013 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Nils Goroll <nils.goroll@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifdef _LONGPATH_REDEFINE_IMPL_H
#error "longpath_redefine_impl.h included twice"
#else
#define _LONGPATH_REDEFINE_IMPL_H

#if ((! defined(_LONGPATH_REDEFINE_PRE_H)) && (! defined(_LONGPATH_REDEFINE_POST_H)))
#error "longpath_redefine_impl.h should not be included directly"
#endif

#include "longpath_config.h"

/*
 * __PRAGMA_REDEFINE_EXTNAME : _first_ redefinition wins (see gcc/c-pragma.c)
 *
 * otherwise: _last_ redefinition wins
 *
 * so longpath_redefine_pre and _post include this dependend upon __PRAGMA_REDEFINE_EXTNAME
 */

/* func -> func64 */
#if !defined(_LP64) && (_FILE_OFFSET_BITS == 64)
 #ifdef __PRAGMA_REDEFINE_EXTNAME
  #pragma redefine_extname        open		openl64
  #pragma redefine_extname        creat		creatl64
  #pragma redefine_extname        stat		statl64
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #pragma redefine_extname        openat	openatl64
#ifdef HAVE_ATTROPEN
   #pragma redefine_extname        attropen	attropenl64
#endif
   #pragma redefine_extname        fstatat	fstatatl64
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #pragma redefine_extname        lstat	lstatl64
  #endif
  #pragma redefine_extname        mkstemp	mkstempl64
  #pragma redefine_extname        mkstemps	mkstempsl64
  #if defined(_GNU_SOURCE)
   #pragma redefine_extname        mkostemp	mkostempl64
   #pragma redefine_extname        mkostemps	mkostempsl64
  #endif
 #else /* __PRAGMA_REDEFINE_EXTNAME */
  #define        open		openl64
  #define        creat		creatl64
  #define        stat(p,b)     	statl64(p,b)
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #define        openat	openatl64
#ifdef HAVE_ATTROPEN
   #define        attropen	attropenl64
#endif
   #define        fstatat	fstatatl64
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #define        lstat	lstatl64
  #endif
  #define        mkstemp	mkstempl64
  #define        mkstemps	mkstempsl64
  #if defined(_GNU_SOURCE)
   #define        mkostemp	mkostempl64
   #define        mkostemps	mkostempsl64
  #endif
 #endif  /* __PRAGMA_REDEFINE_EXTNAME */
#else  /* _LP64 && _FILE_OFFSET_BITS == 64 */
 #ifdef __PRAGMA_REDEFINE_EXTNAME
  #pragma redefine_extname        open	openl
  #pragma redefine_extname        creat	creatl
  #pragma redefine_extname        stat	statl
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #pragma redefine_extname        openat	openatl
#ifdef HAVE_ATTROPEN
   #pragma redefine_extname        attropen	attropenl
#endif
   #pragma redefine_extname        fstatat	fstatatl
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #pragma redefine_extname        lstat	lstatl
  #endif
  #pragma redefine_extname        mkstemp	mkstempl
  #pragma redefine_extname        mkstemps	mkstempsl
  #if defined(_GNU_SOURCE)
   #pragma redefine_extname        mkostemp	mkostempl
   #pragma redefine_extname        mkostemps	mkostempsl
  #endif
 #else /* __PRAGMA_REDEFINE_EXTNAME */
  #define        open		openl
  #define        creat		creatl
  #define        stat(p, b)	statl(p, b)
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #define        openat	openatl
#ifdef HAVE_ATTROPEN
   #define        attropen	attropenl
#endif
   #define        fstatat	fstatatl
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #define        lstat	lstatl
  #endif
  #define        mkstemp	mkstempl
  #define        mkstemps	mkstempsl
  #if defined(_GNU_SOURCE)
   #define        mkostemp	mkostempl
   #define        mkostemps	mkostempsl
  #endif
 #endif  /* __PRAGMA_REDEFINE_EXTNAME */
#endif /* _LP64 && _FILE_OFFSET_BITS == 64 */

/* func64 -> func */
#if defined(_LP64) && defined(_LARGEFILE64_SOURCE)
 #ifdef __PRAGMA_REDEFINE_EXTNAME
  #pragma redefine_extname        open64	openl
  #pragma redefine_extname        creat64	creatl
  #pragma redefine_extname        stat64	statl
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #pragma redefine_extname        openat64	openatl
#ifdef HAVE_ATTROPEN
   #pragma redefine_extname        attropen64	attropenl
#endif
   #pragma redefine_extname        fstatat64	fstatatl
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #pragma redefine_extname        lstat64	lstatl
  #endif
  #pragma redefine_extname        mkstemp64	mkstempl
  #pragma redefine_extname        mkstemps64	mkstempsl
  #if defined(_GNU_SOURCE)
   #pragma redefine_extname        mkostemp64	mkostempl
   #pragma redefine_extname        mkostemps64	mkostempsl
  #endif
 #else /* __PRAGMA_REDEFINE_EXTNAME */
  #define        open64		openl
  #define        creat64	creatl
  #define        stat64		statl
  #if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	  defined(_ATFILE_SOURCE)
   #define        openat64	openatl
#ifdef HAVE_ATTROPEN
   #define        attropen64	attropenl
#endif
   #define        fstatat64	fstatatl
  #endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
  #if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
   #define        lstat64	lstatl
  #endif
  #define        mkstemp64	mkstempl
  #define        mkstemps64	mkstempsl
  #if defined(_GNU_SOURCE)
   #define        mkostemp64	mkostempl
   #define        mkostemps64	mkostempsl
  #endif
 #endif  /* __PRAGMA_REDEFINE_EXTNAME */
#endif  /* _LP64 && _LARGEFILE64_SOURCE */


/* func- > funcl redefinitions for all which have not been done above */

#ifdef __PRAGMA_REDEFINE_EXTNAME
#pragma redefine_extname	link	linkl
#pragma redefine_extname	unlink	unlinkl
#pragma redefine_extname	chown	chownl
#pragma redefine_extname	chmod	chmodl
#pragma redefine_extname	mknod	mknodl
#pragma redefine_extname	mkfifo	mkfifol
#pragma redefine_extname	rmdir	rmdirl
#pragma redefine_extname	mkdir	mkdirl
#pragma redefine_extname	rename	renamel
#pragma redefine_extname	utimes	utimesl
#pragma redefine_extname	lutimes	lutimesl
#pragma redefine_extname	utime	utimel

/* at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#if defined(HAVE_LINKAT)
#pragma redefine_extname	linkat		linkatl
#endif
#pragma redefine_extname	unlinkat	unlinkatl
#if defined(HAVE_SYMLINKAT)
#pragma redefine_extname	symlinkat	symlinkatl
#endif
#if defined(HAVE_READLINKAT)
#pragma redefine_extname	readlinkat	readlinkatl
#endif
#pragma redefine_extname	fchownat	fchownatl
#if defined(HAVE_MKNODAT)
#pragma redefine_extname	mknodat		mknodatl
#endif
#if defined(HAVE_MKFIFOAT)
#pragma redefine_extname	mkfifoat	mkfifoatl
#endif
#pragma redefine_extname	rmdirat		rmdiratl
#if defined(HAVE_MKDIRAT)
#pragma redefine_extname	mkdirat		mkdiratl
#endif
#pragma redefine_extname	renameat	renameatl
#endif

/* times at */
#if defined(__EXTENSIONS__) || \
        (!defined(_STRICT_STDC) && !defined(__XOPEN_OR_POSIX))
        /* || defined(_XPG7) */
#pragma redefine_extname	futimesat	futimesatl
#pragma redefine_extname	utimensat	utimensatl
#endif

/* lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#pragma redefine_extname	symlink		symlinkl
#pragma redefine_extname	readlink	readlinkl
#pragma redefine_extname	lchown		lchownl
#endif

/* mkstemp */
#pragma redefine_extname	mktemp		mktempl
#pragma redefine_extname	mkdtemp		mkdtempl

/* stdlib */
#pragma redefine_extname	opendir		opendirl

/* linux */
#ifdef HAVE_FGETXATTR
#pragma	redefine_extname	getxattr	getxattrl
#pragma	redefine_extname	lgetxattr	lgetxattrl
#pragma	redefine_extname	setxattr	setxattrl
#pragma	redefine_extname	lsetxattr	lsetxattrl
#endif

/* 64 */
#if !defined(_LP64)
#pragma redefine_extname	open64		openl64
#pragma redefine_extname	creat64		creatl64
#pragma redefine_extname	stat64		statl64

/* 64 at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#pragma redefine_extname	openat64		openatl64
#ifdef HAVE_ATTROPEN
#pragma redefine_extname	attropen64		attropenl64
#endif
#pragma redefine_extname	fstatat64		fstatatl64
#endif

/* 64 lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#pragma redefine_extname	lstat64		lstatl64
#endif

/* 64 mkstemp*/
#pragma redefine_extname	mkstemp64		mkstempl64
#pragma redefine_extname	mkstemps64		mkstempsl64
/* 64 mkstemp GNU*/
#if defined(_GNU_SOURCE)
#pragma redefine_extname	mkostemp64		mkostempl64
#pragma redefine_extname	mkostemps64		mkostempsl64
#endif
#endif /* _LP64_ */
#else /* __PRAGMA_REDEFINE_EXTNAME */
#define	link	linkl
#define	unlink	unlinkl
#define	chown	chownl
#define	chmod	chmodl
#define	mknod	mknodl
#define	mkfifo	mkfifol
#define	rmdir	rmdirl
#define	mkdir	mkdirl
#define	rename	renamel
#define	utimes	utimesl
#define	lutimes	lutimesl
#define	utime	utimel

/* at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#if defined(HAVE_LINKAT)
#define	linkat		linkatl
#endif
#define	unlinkat	unlinkatl
#if defined(HAVE_SYMLINKAT)
#define	symlinkat	symlinkatl
#endif
#if defined(HAVE_READLINKAT)
#define	readlinkat	readlinkatl
#endif
#define	fchownat	fchownatl
#if defined(HAVE_MKNODAT)
#define	mknodat		mknodatl
#endif
#if defined(HAVE_MKFIFOAT)
#define	mkfifoat	mkfifoatl
#endif
#define	rmdirat		rmdiratl
#if defined(HAVE_MKDIRAT)
#define	mkdirat		mkdiratl
#endif
#define	renameat	renameatl
#endif

/* times at */
#if defined(__EXTENSIONS__) || \
        (!defined(_STRICT_STDC) && !defined(__XOPEN_OR_POSIX))
        /* || defined(_XPG7) */
#define	futimesat	futimesatl
#define	utimensat	utimensatl
#endif

/* lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#define	symlink		symlinkl
#define	readlink	readlinkl
#define	lchown		lchownl
#endif

/* mkstemp */
#define	mktemp		mktempl
#define	mkdtemp		mkdtempl

/* stdlib */
#define	opendir		opendirl

/* linux */
#ifdef HAVE_FGETXATTR
#define	getxattr	getxattrl
#define	lgetxattr	lgetxattrl
#define	setxattr	setxattrl
#define	lsetxattr	lsetxattrl
#endif

/* 64 */
#if !defined(_LP64)
#define	open64		openl64
#define	creat64		creatl64
#define	stat64		statl64

/* 64 at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#define	openat64		openatl64
#ifdef HAVE_ATTROPEN
#define	attropen64		attropenl64
#endif
#define	fstatat64		fstatatl64
#endif

/* 64 lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#define	lstat64		lstatl64
#endif

/* 64 mkstemp*/
#define	mkstemp64		mkstempl64
#define	mkstemps64		mkstempsl64
/* 64 mkstemp GNU*/
#if defined(_GNU_SOURCE)
#define	mkostemp64		mkostempl64
#define	mkostemps64		mkostempsl64
#endif
#endif /* _LP64_ */
#endif /* __PRAGMA_REDEFINE_EXTNAME */

#include "longpath.h"

#endif /* _LONGPATH_REDEFINE_IMPL_H */
